----------------------------------
--      Locals                  --
----------------------------------

local name = "Timers: BlackWing Lair"
local L = AceLibrary("AceLocale-2.2"):new("BigWigs"..name)
local packNumber = nil

local deathThrottle = nil

----------------------------------
--      Localization            --
----------------------------------
L:RegisterTranslations("enUS", function() return {
	cmd = "TimersBWL",
	Bars_cmd = "bars",
	Bars_name= "Bars",
	Bars_desc= "Toggles showing bars for respawn times.",
	Messages_cmd ="messages",
	Messages_name = "Messages",
	Messages_desc = "Toggles showing messages for respawn times.",

	

	["%s %d respawns in %d sec"] = true,
	["%s %d respawned!"] = true,
	["Respawn: %s %d"] = true,

	-- BWL Triggers
	["Death Talon Hatcher"] = true,
	["Blackwing Taskmaster"] = true,
	["Death Talon Captain"] = true,
	-- BWL Messages
	["Death Talon"] = true,
	["Taskmaster"] = true,
	
} end )
L:RegisterTranslations("deDE", function() return {
	cmd = "TimersBWL",
	Bars_cmd = "bars",
	Bars_name= "Balken",
	Bars_desc= "Zeige Balken",
	Messages_cmd ="messages",
	Messages_name = "Warnungen",
	Messages_desc = "Sende Warnungen",
		
	["%s %d respawns in %d sec"] = "%s %d respawnt in %d Sekunden",
	["%s %d respawned!"] = "%s %s respawnt",
	["Respawn: %s %d"] = "Respawn: %s %d",

	-- BWL Triggers
	["Death Talon Hatcher"] = "Brutw\195\164chter der Todeskrallen",
	["Blackwing Taskmaster"] = "Zuchtmeister der Pechschwingen",
	["Death Talon Captain"] = "Death Talon Captain",  -- Need trans
	-- BWL Messages
	["Death Talon"] = "Brutw\195\164chter",
	["Taskmaster"] = "Zuchtmeister",
	

} end )

local deathTable = {
	-- BWL
	[L["Death Talon Hatcher"]]  = {720, L["Death Talon"], nil},
	[L["Blackwing Taskmaster"]] = {720, L["Taskmaster"], nil},
}


----------------------------------
--  Addon Declaration           --
----------------------------------

BigWigsBWLTimers = BigWigs:NewModule(name)
BigWigsBWLTimers.synctoken = name
BigWigsBWLTimers.zonename = AceLibrary("Babble-Zone-2.2")["Blackwing Lair"]
local razorgore = AceLibrary("Babble-Boss-2.2")["Razorgore the Untamed"]
local vael = AceLibrary("Babble-Boss-2.2")["Vaelastrasz the Corrupt"]
local brood = AceLibrary("Babble-Boss-2.2")["Broodlord Lashlayer"]
BigWigsBWLTimers.enabletrigger = { L["Death Talon Captain"], vael, razorgore,brood }
BigWigsBWLTimers.toggleoptions = { "Bars","Messages" }
BigWigsBWLTimers.revision = tonumber(string.sub("$Revision: 19666 $", 12, -3))

----------------------------------
--      Initialization          --
----------------------------------

function BigWigsBWLTimers:OnEnable()
	packNumber = {}
	deathThrottle = {}
	self:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	
end

----------------------------------
--      Events                  --
----------------------------------

function BigWigsBWLTimers:CHAT_MSG_COMBAT_HOSTILE_DEATH(msg)
	for key, table in pairs(deathTable) do
		if msg == string.format(UNITDIESOTHER, key) then
			self:StartRespawnTimer(table[2], table[1], table[3])
		break
		end
	end
	if msg == string.format(UNITDIESOTHER, "Broodlord Lashlayer") then
	self:TriggerEvent("BigWigs_StartBar", self, "Broodlord Down",60)
	self.core:ToggleModuleActive(self, false)
	end
end

function BigWigsBWLTimers:ZONE_CHANGED_NEW_AREA()
if not UnitIsGhost("player") then
if GetRealZoneText() == AceLibrary("Babble-Zone-2.2")["Blackwing Lair"] then
else
self.core:ToggleModuleActive(self, false)
end
end
end
----------------------------------
--    Utility                   --
----------------------------------

function BigWigsBWLTimers:StartRespawnTimer(message, timer, icon, throttle)
	timer = tonumber(timer)
	if timer == nil then return end

	-- Throttle deaths by 60 seconds.
	if deathThrottle[message] and (deathThrottle[message] + 60) > GetTime() then return end
	deathThrottle[message] = GetTime()

	-- Increment the pack counter after all arguments are checked and we're sure
	-- to start a new respawn timer.
	if not packNumber[message] then packNumber[message] = 0 end
	local nextPack = packNumber[message] + 1
	packNumber[message] = nextPack

	-- Make sure the user wants respawn messages.
	if self.db.profile.Messages then
		self:ScheduleEvent("bwltm"..nextPack.."10secwarn", "BigWigs_Message", timer - 10, string.format(L["%s %d respawns in %d sec"], message, nextPack, 10), "Yellow")
		self:ScheduleEvent("bwltm"..nextPack.."respawned", "BigWigs_Message", timer, string.format(L["%s %d respawned!"], message, nextPack), "Orange")
	end

	-- Make sure the user wants respawn timer bars.
	if self.db.profile.Bars then
		-- Default icon.
		if not icon then icon = "Interface\\Icons\\INV_Misc_Head_Dragon_01" end
		local text = string.format(L["Respawn: %s %d"], message, nextPack)
		self:TriggerEvent("BigWigs_StartBar", self, text, timer, icon)
		self:SetCandyBarOnClick("BigWigsBar "..text, function()
			if IsControlKeyDown() then
				self:TriggerEvent("BigWigs_StopBar", self, text)
				self:CancelScheduledEvent("bwltm"..nextPack.."10secwarn")
				self:CancelScheduledEvent("bwltm"..nextPack.."respawned")
			end
		end)
	end
end

