--[[
Name: School-2.0
Revision: $Rev: 18418 $
Author: Kodewulf (kodewulf@gmail.com)
Website: http://www.wowace.com
Description: Library providing information regarding trainers.
Dependencies: AceLibrary, AceOO-2.0, AceLocale-2.2, AceEvent-2.0
]]

local MAJOR_VERSION = "School-2.0"
local MINOR_VERSION = "$Rev: 18418 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
if not AceLibrary:HasInstance("AceOO-2.0") then error(MAJOR_VERSION .. " requires AceOO-2.0") end 
if not AceLibrary:HasInstance("AceLocale-2.2") then error(MAJOR_VERSION .. " requires AceLocale-2.2") end
if not AceLibrary:HasInstance("AceEvent-2.0") then error(MAJOR_VERSION .. " requires AceEvent-2.0") end

local useBabble = false
local SchoolLib = {}
SchoolLib.lists = {}

if AceLibrary:HasInstance("Gratuity-2.0") then SchoolLib.tooltip = AceLibrary("Gratuity-2.0") end

-------------------------------------------------------------------------------
-- Localization
-------------------------------------------------------------------------------
SchoolLib.lang = AceLibrary("AceLocale-2.2"):new(MAJOR_VERSION)

if (AceLibrary:HasInstance("Babble-Zone-2.2") and (useBabble)) then SchoolLib.bz = AceLibrary("Babble-Zone-2.2") end
if (AceLibrary:HasInstance("Babble-Class-2.2") and (useBabble)) then SchoolLib.bc = AceLibrary("Babble-Class-2.2") end


-------------------------------------------------------------------------------
-- Public
-------------------------------------------------------------------------------
function SchoolLib:GetData()
end

function SchoolLib:GotoSchool()
end

-------------------------------------------------------------------------------
-- Internal
-------------------------------------------------------------------------------
function SchoolLib:Clone(source)
end

function SchoolLib:BuildStudent()
end

function SchoolLib:BuildTeacher()
end

function SchoolLib:BuildSchool()
end

-------------------------------------------------------------------------------
-- Scan student
-------------------------------------------------------------------------------
function SchoolLib:ScanStudentLevel()
end

function SchoolLib:ScanStudentMoney()
end

function SchoolLib:ScanStudentCharacterPoints()
end

function SchoolLib:ScanStudentSkills()
end

function SchoolLib:ScanStudentProfessions()
end

function SchoolLib:ScanStudentSpells()
end

function SchoolLib:ScanStudentTalents()
end

function SchoolLib:ScanStudentTradeskills()
end

-------------------------------------------------------------------------------
-- Scan teacher
-------------------------------------------------------------------------------
function SchoolLib:ScanTeacherTrainer()
end

function SchoolLib:ScanTeacherMerchant()
end

-------------------------------------------------------------------------------
-- GetData
-------------------------------------------------------------------------------
function SchoolLib:GetDataStudent()
end

function SchoolLib:GetDataTeacher()
end

function SchoolLib:GetDataSchool()
end

-------------------------------------------------------------------------------
-- Utility
-------------------------------------------------------------------------------
function SchoolLib:GetTeacherName()
    return UnitName("npc")
end

function SchoolLib:GetTeacherZone()
    local zone = GetZoneText()
    if (self.bz) then zone = self.bz[zone] end
    return zone
end

function SchoolLib:GetTeacherSubZone(teacher)
    local subzone = GetSubZoneText()
    if (self.bz) and (string.len(subzone) > 0) then subzone = self.bz[subzone] end
    return subzone
end

function SchoolLib:GetTeacherType(teacher)
    local result
    
    if (self.tooltip) then 
        self.tooltip:SetUnit("npc")
        result = self.tooltip:GetLine(2)
    end
    
    if (not result) then
        if (not teacher) then teacher = self:GetTeacherName() end
    
        if (not self.lists) then self.lists = {} end
        if (not self.lists.teacher) then self.lists.teacher = {} end
        if (not self.lists.teacher.types) then self.lists.teacher.types = {} end
        self.lists.teacher.types["Aldre"] = "Priest Trainer"
        self.lists.teacher.types["Lotheolan"] = "Priest Trainers"
        self.lists.teacher.types["Daestra"] = "Demon Trainer"
        
        result = self.lists.teacher.types[teacher]
    end

    return result
end

function SchoolLib:NPCIsTeacher(npc)
    local result = true
    if (not self.lists) then self.lists = {} end
    if (not self.lists.teacher) then self.lists.teacher = {} end
    if (not self.lists.teacher.teachers) then self.lists.teacher.teachers = {} end
    if (not self.lists.teacher.teachers.types) then self.lists.teacher.teachers.types = {} end
    if (not self.lists.teacher.teachers.names) then self.lists.teacher.teachers.names = {} end
    
    self.lists.teacher.teachers.types["Demon Trainer"] = true
    
    self.lists.teacher.teachers.names["Daestra"] = true
    
    if (not self.lists.teacher.teachers.types[npc.type]) then result = false end
    if (not result) and (self.lists.teacher.teachers.names[npc.name]) then result = true end
    
    return result
end

-------------------------------------------------------------------------------
-- Library
-------------------------------------------------------------------------------
local function activate(self, oldLib, oldDeactivate)
    SchoolLib = self
    if (oldLib) then
        oldLib:CancelAllScheduledEvents()
        oldLib:UnregisterAllEvents()
    else
    
    
    end
    
    if (oldDeactivate) then oldDeactivate(oldLib) end
end

local function external(self, major, instance)
    if (major == "AceEvent-2.0") then
        AceEvent = instance
        AceEvent:embed(self)
        self:UnregisterAllEvents()
        self:CancelAllScheduledEvents()
        if (AceEvent:IsFullyInitialized()) then
            self:AceEvent_FullyInitialized()
        else
            self:RegisterEvent("AceEvent_FullyInitialized", "AceEvent_FullyInitialized", true)
        end
    end
end

-------------------------------------------------------------------------------
-- Events
-------------------------------------------------------------------------------
function SchoolLib:AceEvent_FullyInitialized()
    self:RegisterEvent("TRAINER_SHOW", "Event_Trainer", 2)
    self:RegisterEvent("TRAINER_CLOSED", "Event_Trainer", 2)
    
    self:RegisterEvent("MERCHANT_SHOW", "Event_Merchant", 2)
    self:RegisterEvent("MERCHANT_CLOSED", "Event_Merchant", 2)
    
    self:RegisterEvent("CHARACTER_POINTS_CHANGED", "Event_CharacterPoints", 2)
    self:RegisterEvent("CHAT_MSG_SKILL", "Event_ChatMsgSkill", 5)
    self:RegisterEvent("SPELLS_CHANGED", "Event_Spells", 5)
    self:RegisterEvent("PLAYER_LEVEL_UP", "Event_PlayerLevelUp")

    self.inCombat = false
    self:RegisterEvent("PLAYER_ENTERING_WORLD", "Event_PlayerEnteringWorld")
    self:RegisterEvent("PLAYER_REGEN_DISABLED", function() self.inCombat = true end)
    self:RegisterEvent("PLAYER_REGEN_ENABLED", function() self.inCombat = false end)
    
    self:RegisterEvent("School_Build", "Event_SchoolBuild")
    self:RegisterEvent("School_Teacher_Changed", "Event_SchoolTeacherChanged", 5)
    self:RegisterEvent("School_Student_Changed", "Event_SchoolStudentChanged", 5)
end

function SchoolLib:Event_Trainer()
    self:ScanTeacherTrainer()
end

function SchoolLib:Event_Merchant()
    self:ScanTeacherMerchant()
end

function SchoolLib:Event_PlayerLevelUp()
    self:ScanStudentLevel()
end

function SchoolLib:Event_Money()
    self:ScanStudentMoney()
end

function SchoolLib:Event_CharacterPoints()
    self:ScanStudentCharacterPoints()
end

function SchoolLib:Event_ChatMsgSkill()
    self:ScanStudentSkills()
end

function SchoolLib:Event_Spells()
    self:ScanStudentSpells()
end

function SchoolLib:Event_PlayerEnteringWorld()
end

function SchoolLib:Event_SchoolBuild()
end

function SchoolLib:Event_SchoolTeacherChanged()
end

function SchoolLib:Event_SchoolStudentChanged()
end

-------------------------------------------------------------------------------
-- Load
-------------------------------------------------------------------------------
AceLibrary:Register(SchoolLib, MAJOR_VERSION, MINOR_VERSION, activate, nil, external)
